/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.field.shape;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.field.shape.preview.ShapePropertyMappingPanel;
import nl.cloudfarming.client.util.actions.OpenCloseHandler;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@MultiViewElement.Registration(displayName = "#LBL_Shape_mapping",
iconBase = "nl/cloudfarming/client/field/shape/file_extension_shp.png",
mimeType = ShapeDataObject.MIMETYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "ShapeMapping",
position = 1000)
@NbBundle.Messages("LBL_Shape_mapping=Mapping")
public final class ShapeMappingElement extends JPanel implements MultiViewElement {

    private ShapeDataObject obj;
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private ASBodyPanel content;
    private ShapeMappingPanel mappingPanel = new ShapeMappingPanel();
    private static final String SHAPE_FILE_ICON_PATH = "nl/cloudfarming/client/field/shape/file_extension_shp32.png";

    /**
     * Creates new form ShapeMappingElement
     */
    public ShapeMappingElement(Lookup lkp) {
        obj = lkp.lookup(ShapeDataObject.class);
        assert obj != null;

        setLayout(new BorderLayout());

        //FieldVisualPanel fieldVisualPanel = new FieldVisualPanel(obj.getLookup().lookup(Field.class));
        // Shape selectorPanel
        List<String> valueAttributes = obj.getIntegerAttributes();
        if (valueAttributes != null && !valueAttributes.isEmpty() && !valueAttributes.contains("")) {
            valueAttributes.add("");
        }

        List<String> nameAttributes = obj.getStringAttributes();
        if (nameAttributes != null && !nameAttributes.isEmpty() && !nameAttributes.contains("")) {
            nameAttributes.add("");
        }

        if (valueAttributes != null && !valueAttributes.isEmpty()
                && nameAttributes != null && !nameAttributes.contains(valueAttributes.get(0))) {
            nameAttributes.addAll(valueAttributes);
        }
        mappingPanel.getShapePropertyMappingPanel().setAvailableNameProperties(nameAttributes);
        mappingPanel.getShapePropertyMappingPanel().setAvailableValueProperties(valueAttributes);

        mappingPanel.getShapePropertyMappingPanel().setSelectedNameProperty(obj.getNameProperty());
        mappingPanel.getShapePropertyMappingPanel().setSelectedValueProperty(obj.getValueProperty());

        mappingPanel.getShapePropertyMappingPanel().addPropertyChangeListener(new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (ShapePropertyMappingPanel.PROP_COMBOBOX_MAPPING_NAME.equals(evt.getPropertyName())){
                    obj.setNameProperty((String)evt.getNewValue());
                    for (Shape shape:obj.getShapes()){
                        shape.setNameFromAttribute((String)evt.getNewValue());
                    }
                }
                if (ShapePropertyMappingPanel.PROP_COMBOBOX_MAPPING_VALUE.equals(evt.getPropertyName())){
                    obj.setValueProperty((String)evt.getNewValue());
                    for (Shape shape:obj.getShapes()){
                        shape.setValueFromAttribute((String)evt.getNewValue());
                    }                    
                }                
            }
        });

        //
        // add binding for content type radio box
        //
        mappingPanel.setContentType(obj.getContentType());
        mappingPanel.addPropertyChangeListener(ShapeMappingPanel.PROP_CONTENT_TYPE, new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                obj.setContentType((int)evt.getNewValue());
            }
        });

        content = new ASBodyPanel(mappingPanel);
        content.setIcon(ImageUtilities.loadImageIcon(SHAPE_FILE_ICON_PATH, true));
        content.setTitle(Bundle.LBL_Shape_mapping());


        add(content);

    }

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
        OpenCloseHandler.Event.triggerOpen(obj.getNodeDelegate());
    }

    @Override
    public void componentClosed() {
        OpenCloseHandler.Event.triggerClose(obj.getNodeDelegate());
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private void updateName() {
        if (callback != null) {
            FileObject shapeFile = obj.getLookup().lookup(FileObject.class);
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(shapeFile.getName());
            tc.setDisplayName(shapeFile.getName());
            tc.setName(getName());
            tc.setToolTipText(getToolTipText());
        }
    }
}
