/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.field.shape;

import java.awt.Color;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import nl.cloudfarming.client.util.swing.FormRow;
import org.openide.util.NbBundle;

/**
 * Form row for a single shape property/attribute mapping.
 * 
 * @author johan
 */
@NbBundle.Messages({
    "Shape property mapping formrow name label=Name: ",
    "Shape property mapping formrow name tooltip=Name",
    "Shape property mapping formrow value label=Value: ",
    "Shape property mapping formrow value tooltip=Value",
    "Shape property mapping formrow no properties message=None found"
})
public class ShapePropertyMappingFormRow extends FormRow {
    private final DefaultComboBoxModel<String> propertiesModel;
    private final JComboBox<String> propertiesComboBox;

    private ShapePropertyMappingFormRow(boolean required, String labelText, String labelTooltipText, JComboBox<String> component) {
        super(required, labelText, labelTooltipText, component);
        this.propertiesComboBox = component;
        this.propertiesComboBox.setBackground(Color.WHITE);
        this.propertiesModel = (DefaultComboBoxModel<String>) component.getModel();
    }

    public JComboBox getPropertiesComboBox() {
        return propertiesComboBox;
    }

    public String getSelectedProperty(){
        return (String) propertiesModel.getSelectedItem();
    }
    
    public void setSelectedProperty(String property){
        propertiesModel.setSelectedItem(property);
    }
    
    public void setAvailableProperties(List<String> properties) {
        propertiesModel.removeAllElements();
        propertiesComboBox.removeAllItems();
        
        if (properties == null || properties.isEmpty()) {
            propertiesModel.addElement(Bundle.Shape_property_mapping_formrow_no_properties_message());
            propertiesComboBox.setEnabled(false);
        } else {
            propertiesComboBox.setEnabled(true);
            for (String property : properties) {
                propertiesModel.addElement(property);
            }
            if (propertiesModel.getIndexOf("") >= 0) {
                propertiesModel.setSelectedItem("");
            }
        }
    }
    
    static ShapePropertyMappingFormRow create(boolean required, String labelText, String labelTooltipText) {
        return new ShapePropertyMappingFormRow(required, labelText, labelTooltipText, new JComboBox<>(new DefaultComboBoxModel<String>()));
    }
    
    public static ShapePropertyMappingFormRow createNameRow(boolean required) {
        return create(required, Bundle.Shape_property_mapping_formrow_name_label(), Bundle.Shape_property_mapping_formrow_name_tooltip());
    }

    public static ShapePropertyMappingFormRow createValueRow(boolean required) {
        return create(required, Bundle.Shape_property_mapping_formrow_value_label(), Bundle.Shape_property_mapping_formrow_value_tooltip());
    }
}
