/*
 * Copyright (c) 2012 AgroSense.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    AgroSense - initial API and implementation and/or initial documentation
 */
package nl.cloudfarming.client.field.shape;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.field.shape.preview.ShapePropertyMappingPanel;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * 
 * @author johan
 */
@NbBundle.Messages({
    "ShapePropertyMappingWizardPanel1 step name=Configure mapping",
    "ShapePropertyMappingWizardPanel1 name attribute required=The name attribute is required",
    "ShapePropertyMappingWizardPanel1 value attribute required=The value attribute is required"
})
public class ShapePropertyMappingWizardPanel1 implements WizardDescriptor.Panel<WizardDescriptor> {

    public static final String PROP_DATA_OBJECT = "dataObject";
    public static final String PROP_NAME_PROPERTY = "nameProperty";
    public static final String PROP_VALUE_PROPERTY = "valueProperty";
    public static final String PROP_SAVE_TO_REGISTRY = "saveToRegistry";
    
    /**
     * The visual component that displays this panel. If you need to access the
     * component from this class, just use getComponent().
     */
    private ShapePropertyMappingVisualPanel1 visualComponent;
    private ASBodyPanel component;
    private final Set<ChangeListener> listeners = new HashSet<>(1);
    private WizardDescriptor wizardDescriptor;
    private ShapeDataObject dataObject;
    private String valueProperty;

    // Get the visual component for the panel. In this template, the component
    // is kept separate. This can be more efficient: if the wizard is created
    // but never displayed, or not all panels are displayed, it is better to
    // create only those which really need to be visible.
    @Override
    public ASBodyPanel getComponent() {
        if (component == null) {
            visualComponent = new ShapePropertyMappingVisualPanel1();
            component = new ASBodyPanel(visualComponent);
            component.setIcon(ImageUtilities.loadImageIcon("nl/cloudfarming/client/field/shape/file_extension_shp32.png", true));
            component.setTitle(Bundle.ShapePropertyMappingWizardPanel1_step_name());
            component.setName(Bundle.ShapePropertyMappingWizardPanel1_step_name());

            visualComponent.addPropertyChangeListener(new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    switch (evt.getPropertyName()) {
                        case ShapePropertyMappingPanel.PROP_COMBOBOX_MAPPING_VALUE:
                            valueProperty = ((String) evt.getNewValue()).trim();
                            fireChangeEvent();
                            break;
                    }
                }
            });
            // default to checked:
            visualComponent.getSaveToRegistryCheckBox().setSelected(true);
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        // Show no Help button for this panel:
        return HelpCtx.DEFAULT_HELP;
        // If you have context help:
        // return new HelpCtx("help.key.here");
    }

    @Override
    public boolean isValid() {
        // If it is always OK to press Next or Finish, then:
//        return true;
        // If it depends on some condition (form filled out...) and
        // this condition changes (last form field filled in...) then
        // use ChangeSupport to implement add/removeChangeListener below.
        // WizardDescriptor.ERROR/WARNING/INFORMATION_MESSAGE will also be useful.
        String errorMsg = null;
        try {
            validate();
        } catch (WizardValidationException e) {
            errorMsg = e.getLocalizedMessage();
        } finally {
            wizardDescriptor.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, errorMsg);
            return errorMsg == null;
        }
    }
    
    public void validate() throws WizardValidationException {        
        if (valueProperty == null || valueProperty.isEmpty()) {
            throw new WizardValidationException(null, Bundle.ShapePropertyMappingWizardPanel1_value_attribute_required(), null);
        }
    }
    
    @Override
    public void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }
    
    @Override
    public void readSettings(WizardDescriptor wiz) {
        // use wiz.getProperty to retrieve previous panel state
        wizardDescriptor = wiz;
        dataObject = (ShapeDataObject) wiz.getProperty(PROP_DATA_OBJECT);
        component.setTitle(dataObject.getName());
        
        // FIXME: duplicate/similar code in ShapeDataObject (autodetect mapping), ShapeMappingElement, and this:
        // create central methods to fill the 2 prop dropdowns
        final List<String> valueProperties = new LinkedList<>();
        valueProperties.add("");
        if (!dataObject.getIntegerAttributes().isEmpty()) {
            valueProperties.addAll(dataObject.getIntegerAttributes());
        }

        visualComponent.getValueMappingFormRow().setAvailableProperties(valueProperties);
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        // use wiz.putProperty to remember current panel state
        wiz.putProperty(PROP_VALUE_PROPERTY, valueProperty);
        wiz.putProperty(PROP_SAVE_TO_REGISTRY, visualComponent.getSaveToRegistryCheckBox().isSelected());
    }
}
