/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field.shape.preview;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.JPanel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.field.shape.ShapePropertyMappingFormRow;

/**
 * Allows the user to select the properties from a shape file which represent
 * the 'Name' and 'Value' of a shape.
 *
 * @author Gerben Feenstra
 * @author johan
 */
public class ShapePropertyMappingPanel extends JPanel {
    
    public static final String PROP_COMBOBOX_MAPPING_NAME = "nameMappingBox";
    public static final String PROP_COMBOBOX_MAPPING_VALUE = "valueMappingBox";
    
    private final ShapePropertyMappingFormRow nameRow = ShapePropertyMappingFormRow.createNameRow(false);
    private final ShapePropertyMappingFormRow valueRow = ShapePropertyMappingFormRow.createValueRow(false);

    public ShapePropertyMappingPanel() {
        this.setLayout(new MigLayout("wrap 1, insets 0 8 0 0"));
        this.setBackground(Color.white);

        this.add(nameRow);
        this.add(valueRow);
        
        nameRow.getPropertiesComboBox().addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                firePropertyChange(PROP_COMBOBOX_MAPPING_NAME, null, nameRow.getSelectedProperty());
            }
        });
        
        valueRow.getPropertiesComboBox().addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                firePropertyChange(PROP_COMBOBOX_MAPPING_VALUE, null, valueRow.getSelectedProperty());
            }
        });
    }

    public  void setAvailableNameProperties(List<String> properties) {
        nameRow.setAvailableProperties(properties);
    }
    
    public  void setAvailableValueProperties(List<String> properties) {
        valueRow.setAvailableProperties(properties);
    }

    public void setSelectedNameProperty(String property){
        nameRow.setSelectedProperty(property);
    }
    
    public void setSelectedValueProperty(String property){
        valueRow.setSelectedProperty(property);
    }
}
